local VERSION = tonumber(("$Revision: 33070 $"):match("%d+"))

local PitBull = PitBull
local PitBull_ComboPoints = PitBull:NewModule("ComboPoints", "AceEvent-2.0", "AceConsole-2.0")
local self = PitBull_ComboPoints
if PitBull.revision < VERSION then
	PitBull.version = "r" .. VERSION
	PitBull.revision = VERSION
	PitBull.date = ("$Date: 2007-04-19 07:13:19 -0400 (Thu, 19 Apr 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local DogTag = AceLibrary("DogTag-1.0")

local new, del, newHash = PitBull.new, PitBull.del, PitBull.newHash
local newFrame, delFrame = PitBull.newFrame, PitBull.delFrame

PitBull_ComboPoints:RegisterPitBullChildFrames('comboText', 'comboPoints')

function PitBull_ComboPoints:OnEnable()
	local _, class = UnitClass("player")
	if class == "DRUID" then
		self:RegisterBucketEvent("UNIT_DISPLAYPOWER", 0.05)
	end
	self:RegisterEvent("PLAYER_COMBO_POINTS", "UpdateComboPoints")
end

function PitBull_ComboPoints:UpdateComboPoints()
	local frame = PitBull:GetUnitFrame("target")
	if frame and frame.comboPoints then
		local p = GetComboPoints()
		for i = 1, 5 do
			if p >= i then
				frame.comboPoints["combo"..i]:Show()
			else
				frame.comboPoints["combo"..i]:Hide()
			end
		end
		frame.comboPoints:Show()
	end
end

function PitBull_ComboPoints:UNIT_DISPLAYPOWER(units)
	if units.player then
		local frame = PitBull:GetUnitFrame("target")
		if frame and frame.comboText then
			DogTag:RegisterFontString(frame.comboText, "target", UnitPowerType("player") == 3 and "[combos:hidezero]" or "")
		end
	end
end

function PitBull_ComboPoints:OnUpdateFrame(unit, frame)
	if unit == "target" then
		self:UpdateComboPoints()
	end
end

function PitBull_ComboPoints:OnPopulateUnitFrame(unit, frame)
	if unit == "target" then
		local comboText = newFrame("FontString", frame.overlay, "OVERLAY")
		frame.comboText = comboText
		local font, fontsize = PitBull:GetFont()
		comboText:SetFont( font, fontsize )
		comboText:SetShadowColor( 0, 0, 0, 1)
		comboText:SetShadowOffset( 0.8, -0.8 )
		comboText:SetNonSpaceWrap(false)

		DogTag:RegisterFontString(frame.comboText, "target", UnitPowerType("player") == 3 and "[combos:hidezero]" or "")
	
		local comboPoints = newFrame("Frame", frame.overlay, "OVERLAY")
		frame.comboPoints = comboPoints
		for i = 1, 5 do
			local combo = newFrame("Texture", frame.comboPoints, "OVERLAY")
			comboPoints["combo"..i] = combo
		end		
	end
end

function PitBull_ComboPoints:OnUpdateFont(font, fontsize)
	for _,frame in PitBull:IterateUnitFrames() do
		if frame.comboText then
			frame.comboText:SetFont( font, fontsize)
		end
	end
end

function PitBull_ComboPoints:OnClearUnitFrame(unit, frame)
	if frame.comboText then
		frame.comboText = delFrame(frame.comboText)
	end
	if frame.comboPoints then
		for i = 1, 5 do
			frame.comboPoints["combo"..i] = delFrame( frame.comboPoints["combo"..i] )
		end
		frame.comboPoints = delFrame(frame.comboPoints)
	end
end

function PitBull_ComboPoints:OnUnknownLayout(unit, frame, combo)
	if frame.comboText and frame.manaBar and combo == "comboText" then
		frame.comboText:SetPoint("CENTER", frame.manaBar, "CENTER")
	end
	if frame.comboPoints and combo == "comboPoints" then
		frame.comboPoints:SetPoint( "BOTTOMRIGHT", frame, "BOTTOMRIGHT", -5, 0)
		frame.comboPoints:SetWidth( 50 )
		frame.comboPoints:SetHeight( 10 )
		for i = 1, 5 do
			local combo = frame.comboPoints["combo"..i]
			combo:SetWidth( 10 )
			combo:SetHeight( 10 )
			combo:SetTexture("Interface\\AddOns\\PitBull\\ComboPoints\\combo.tga")
			if i == 1 then
				combo:SetPoint("RIGHT", frame.comboPoints, "RIGHT")
			else
				combo:SetPoint("RIGHT", frame.comboPoints["combo"..(i-1)], "LEFT")
			end			
		end
	end
end
