local VERSION = tonumber(("$Revision: 33341 $"):match("%d+"))

local PitBull = PitBull
local PitBull_Health = PitBull:NewModule("Health", "AceEvent-2.0", "AceConsole-2.0")
local self = PitBull_Health
if PitBull.revision < VERSION then
	PitBull.version = "r" .. VERSION
	PitBull.revision = VERSION
	PitBull.date = ("$Date: 2007-04-21 15:42:11 -0400 (Sat, 21 Apr 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local DogTag = AceLibrary("DogTag-1.0")

local new, del, newHash = PitBull.new, PitBull.del, PitBull.newHash
local newFrame, delFrame = PitBull.newFrame, PitBull.delFrame

local UnitHealth = UnitHealth
local UnitHealthMax = UnitHealthMax
local UnitClass = UnitClass

PitBull_Health:RegisterPitBullChildFrames("healthBar")

local unitsShown = {}

function PitBull_Health:OnInitialize()
	self.db = PitBull:AcquireDBNamespace("Health")
	PitBull:RegisterDefaults("Health", "profile", {
		["**"] = {
			classColor = true,
			classColorPvP = false,
			hostilityColor = false,
			hostilityColorNPCs = false,
			deficit = false,
			reverse = false,
			customColor = false,
			customColors = {1,1,1},
			customBGColor = false,
			customBGColors = {11/255, 11/255, 11/255},
		},
		focus = {
			hostilityColor = true,
		},
		target = {
			hostilityColor = true,
		},
		targettarget = {
			hostilityColor = true,
		},
		targettargettarget = {
			hostilityColor = true,
		},
		focustarget = {
			hostilityColor = true,
		},
		partytarget = {
			hostilityColor = true,
		},
		partypettarget = {
			hostilityColor = true,
		},
		raidtarget = {
			hostilityColor = true,
		},
		pet = {
			happinessColor = true,
		},
	})
end

function PitBull_Health:OnEnable(first)
	self:RegisterBucketEvent({"UNIT_HEALTH", "UNIT_MAXHEALTH" }, 0.05, "UNIT_HEALTH")
	self:RegisterBucketEvent("UNIT_FACTION", 0.05, "UNIT_FACTION")
end

function PitBull_Health:UNIT_HEALTH(units)
	for unit in pairs(units) do
		local frame = PitBull:GetUnitFrame(unit)
		if frame and unitsShown[unit] then
			self:UpdateHealth(unit, frame)
			self:UpdateHealthbarColor(unit, frame)
		end
	end
end

function PitBull_Health:UNIT_FACTION(units)
	for unit in pairs(units) do
		local frame = PitBull:GetUnitFrame(unit)
		if frame and unitsShown[unit] then
			self:UpdateHealthbarColor(unit, frame)
		end
	end
end

function PitBull_Health:UpdateHealth(unit, frame)
	local db = self.db.profile[frame.group]
	
	local max = UnitHealthMax(unit)
	if max > 0 then
		local value = UnitHealth(unit)
		local perc = value / max
		if db.deficit ~= db.reverse then
			perc = 1 - perc
		end
		frame.healthBar:SetValue(perc)
	end
end

local _,class = UnitClass("player")
function PitBull_Health:UpdateHealthbarColor(unit, frame)
	local db = self.db.profile[frame.group]
	
	local r, g, b
	if UnitIsDeadOrGhost(unit) then
		r, g, b = unpack(PitBull.colorConstants.dead)
	elseif not UnitIsConnected(unit) then
		r, g, b = unpack(PitBull.colorConstants.disconnected)
	elseif UnitIsTapped(unit) and not UnitIsTappedByPlayer(unit) then
		r, g, b = unpack(PitBull.colorConstants.tapped)
	elseif UnitIsPlayer(unit) then
		if db.classColor and (db.classColorPvP or UnitIsFriend("player", unit)) then
			local _,class = UnitClass(unit)
			r, g, b = unpack(PitBull.colorConstants[class or "WARRIOR"])
		elseif db.hostilityColor then
			if UnitCanAttack(unit, "player") then
				-- they can attack me
				if UnitCanAttack("player", unit) then
					-- and I can attack them
					r, g, b = unpack(PitBull.colorConstants.hostile)
				else
					-- but I can"t attack them
					r, g, b = unpack(PitBull.colorConstants.civilian)
				end
			elseif UnitCanAttack("player", unit) then
				-- they can"t attack me, but I can attack them
				r, g, b = unpack(PitBull.colorConstants.neutral)
			elseif UnitIsFriend("player", unit) then
				-- on my team
				r, g, b = unpack(PitBull.colorConstants.friendly)
			else
				-- either enemy or friend, no violence
				r, g, b = unpack(PitBull.colorConstants.civilian)
			end
		elseif db.customColor then
			r, g, b = unpack(db.customColors)
		end
	elseif UnitIsUnit(unit, "pet") and db.happinessColor then
		if class == "HUNTER" then
			local happy = GetPetHappiness()
			if happy == 3 then
				r, g, b = unpack(PitBull.colorConstants.petHappy)
			elseif happy == 2 then
				r, g, b = unpack(PitBull.colorConstants.petNeutral)
			elseif happy == 1 then
				r, g, b = unpack(PitBull.colorConstants.petAngry)
			end
		end
	elseif db.hostilityColor or db.hostilityColorNPCs then
		local reaction = UnitReaction(unit, "player")
		if reaction then
			if reaction >= 5 then
				r, g, b = unpack(PitBull.colorConstants.friendly)
			elseif reaction == 4 then
				r, g, b = unpack(PitBull.colorConstants.neutral)
			else
				r, g, b = unpack(PitBull.colorConstants.hostile)
			end
		else
			r, g, b = unpack(PitBull.colorConstants.unknown)
		end
	elseif db.customColor then
		r, g, b = unpack(db.customColors)
	end
	if not r then
		local perc = UnitHealth(unit) / UnitHealthMax(unit)
		b = 0
		if perc > 0.5 then
			g = 1
			r = (1 - perc) * 2
		else
			r = 1
			g = perc * 2
		end
	end
	local br, bg, bb
	if db.customBGColor then
		br, bg, bb = unpack(db.customBGColors)
	else
		br, bg, bb = (r+0.2)/3, (g+0.2)/3, (b+0.2)/3
	end
	if db.reverse then
		frame.healthBar:SetStatusBarColor(br, bg, bb, 1)
		frame.healthBar.bg:SetVertexColor(r, g, b, 1)
	else
		frame.healthBar:SetStatusBarColor(r, g, b, 1)
		frame.healthBar.bg:SetVertexColor(br, bg, bb, 1)
	end
end

function PitBull_Health:OnUpdateFrame(unit, frame)
	if unitsShown[unit] then
		self:UpdateHealth(unit, frame)
		self:UpdateHealthbarColor(unit, frame)
	end
end

function PitBull_Health:OnPopulateUnitFrame(unit, frame)
	local texture = PitBull:GetStatusBarTexture()
	local font, fontsize = PitBull:GetFont()
	
	local healthBar = newFrame("StatusBar", frame)
	frame.healthBar = healthBar
	healthBar:SetMinMaxValues(0, 1)
	healthBar:SetValue(1)
	healthBar:SetStatusBarTexture(texture)
	healthBar:SetStatusBarColor(0, 1, 0, 1)

	local healthBarBG = newFrame("Texture", healthBar, "BACKGROUND")
	frame.healthBar.bg = healthBarBG
	healthBarBG:SetTexture(texture)
	healthBarBG:SetVertexColor(0, 0.5, 0, 1)
	healthBarBG:ClearAllPoints()
	healthBarBG:SetAllPoints(healthBar)
end

function PitBull_Health:OnClearUnitFrame(unit, frame)
	frame.healthBar.bg = delFrame(frame.healthBar.bg)
	frame.healthBar = delFrame(frame.healthBar)
end

function PitBull_Health:OnUpdateStatusBarTexture(texture)
	for _,frame in PitBull:IterateUnitFrames() do
		frame.healthBar:SetStatusBarTexture(texture)
		frame.healthBar.bg:SetTexture(texture)
	end
end

function PitBull_Health:OnUnknownLayout(unit, frame, name)
	frame.healthBar:SetPoint("TOPLEFT", frame, "TOPLEFT", 5, -5)
	frame.healthBar:SetPoint("BOTTOMRIGHT", frame, "RIGHT", -5, 0)
end

function PitBull_Health:OnUpdateLayout(unit, frame)
	local old = not unitsShown[unit]
	local now = not frame.healthBar or not frame.healthBar:IsShown()
	if old == now then
		return
	end
	unitsShown[unit] = not now
	if not now then
		self:OnUpdateFrame(unit, frame)
	end
end

local function getClassColor(group)
	return PitBull_Health.db.profile[group].classColor
end
local function setClassColor(group, value)
	PitBull_Health.db.profile[group].classColor = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Health:UpdateHealthbarColor(unit, frame)
	end
end
local function getClassColorPvP(group)
	return PitBull_Health.db.profile[group].classColorPvP
end
local function setClassColorPvP(group, value)
	PitBull_Health.db.profile[group].classColorPvP = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Health:UpdateHealthbarColor(unit, frame)
	end
end
local function getHostilityColor(group)
	return PitBull_Health.db.profile[group].hostilityColor
end
local function setHostilityColor(group, value)
	PitBull_Health.db.profile[group].hostilityColor = value

	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Health:UpdateHealthbarColor(unit, frame)
	end
end

local function getHostilityColorNPCs(group)
	return PitBull_Health.db.profile[group].hostilityColorNPCs
end
local function setHostilityColorNPCs(group, value)
	PitBull_Health.db.profile[group].hostilityColorNPCs = value

	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Health:UpdateHealthbarColor(unit, frame)
	end
end

local function getDeficit(group)
	return PitBull_Health.db.profile[group].deficit
end
local function setDeficit(group, value)
	PitBull_Health.db.profile[group].deficit = value

	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Health:UpdateHealth(unit, frame)
	end
end
local function getReverse(group)
	return PitBull_Health.db.profile[group].reverse
end
local function setReverse(group, value)
	PitBull_Health.db.profile[group].reverse = value

	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Health:UpdateHealth(unit, frame)
		PitBull_Health:UpdateHealthbarColor(unit, frame)
	end
end

local function GetPetHappiness()
	return PitBull_Health.db.profile.pet.happinessColor
end

local function SetPetHappiness(value)
	PitBull_Health.db.profile.pet.happinessColor = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Health:UpdateHealth(unit, frame)
		PitBull_Health:UpdateHealthbarColor(unit, frame)
	end
end

local function GetCustomColorToggle(group)
	return PitBull_Health.db.profile[group].customColor
end

local function SetCustomColorToggle(group, value)
	PitBull_Health.db.profile[group].customColor = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Health:UpdateHealth(unit, frame)
		PitBull_Health:UpdateHealthbarColor(unit, frame)
	end
end

local function GetCustomColor(group)
	return unpack(PitBull_Health.db.profile[group].customColors)
end

local function SetCustomColor(group, r, g, b)
	PitBull_Health.db.profile[group].customColors = {r, g, b}
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Health:UpdateHealth(unit, frame)
		PitBull_Health:UpdateHealthbarColor(unit, frame)
	end
end

local function GetCustomBGColorToggle(group)
	return PitBull_Health.db.profile[group].customBGColor
end

local function SetCustomBGColorToggle(group, value)
	PitBull_Health.db.profile[group].customBGColor = value

	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Health:UpdateHealth(unit, frame)
		PitBull_Health:UpdateHealthbarColor(unit, frame)
	end
end

local function GetCustomBGColor(group)
	return unpack(PitBull_Health.db.profile[group].customBGColors)
end

local function SetCustomBGColor(group, r, g, b)
	PitBull_Health.db.profile[group].customBGColors = {r, g, b}

	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Health:UpdateHealth(unit, frame)
		PitBull_Health:UpdateHealthbarColor(unit, frame)
	end
end

local function GetCustomBGColorToggle(group)
	return PitBull_Health.db.profile[group].customBGColor
end

local function SetCustomBGColorToggle(group, value)
	PitBull_Health.db.profile[group].customBGColor = value

	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Health:UpdateHealth(unit, frame)
		PitBull_Health:UpdateHealthbarColor(unit, frame)
	end
end

local function GetCustomBGColor(group)
	return unpack(PitBull_Health.db.profile[group].customBGColors)
end

local function SetCustomBGColor(group, r, g, b)
	PitBull_Health.db.profile[group].customBGColors = {r, g, b}

	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Health:UpdateHealth(unit, frame)
		PitBull_Health:UpdateHealthbarColor(unit, frame)
	end
end

PitBull_Health:RegisterPitBullOptionsMethod(function(group)
	local hostility, petOptions = nil, nil
	if group == "focus" or group:find("target$") then
		hostility = true
	elseif group == "pet" then
		petOptions = true
	end
	return {
		name = "Health bar",
		desc = "Health bar display options.",
		type = "group",
		args = {
			classColor = {
				type = "toggle",
				name = "Color by class",
				desc = "Color the health bar by unit class.",
				get = getClassColor,
				set = setClassColor,
				passValue = group,
			},
			classColorPvP = hostility and {
				type = "toggle",
				name = "Color by class for PvP enemies",
				desc = "Color the health bar for PvP enemies by unit class.",
				get = getClassColorPvP,
				set = setClassColorPvP,
				passValue = group,
			},
			hostilityColor = hostility and {
				type = "toggle",
				name = "Color by hostility",
				desc = "Color the health bar by hostility. Note that this takes precedence after color by class.",
				get = getHostilityColor,
				set = setHostilityColor,
				passValue = group,
			},
			hostilityColorNPCs = hostility and {
				type = "toggle",
				name = "Color NPCs by hostility",
				desc = "Color the health bar by hostility for NPCs. Note that this takes precedence after color by class.",
				get = getHostilityColorNPCs,
				set = setHostilityColorNPCs,
				passValue = group,
			},
			customColor = {
				type = "group",
				name = "Custom color",
				desc = "Set a custom color for the health bar. Note that this is the least priority after all the color options.",
				args = {
					Enable = {
						type = "toggle",
						name = "Enable",
						desc = "Enable using a custom color for the health bar.",
						get = GetCustomColorToggle,
						set = SetCustomColorToggle,
						passValue = group,
					},
					setColor = {
						type = "color",
						name = "Color",
						desc = "Set the custom color for the health bar.",
						get = GetCustomColor,
						set = SetCustomColor,
						disabled = function() return not GetCustomColorToggle(group) end,
						passValue = group,
					},
				},
			},
			customBGColor = {
				type = "group",
				name = "Custom background color",
				desc = "Set a custom background color for the health bar.",
				args = {
					Enable = {
						type = "toggle",
						name = "Enable",
						desc = "Enable using a custom background color for the health bar.",
						get = GetCustomBGColorToggle,
						set = SetCustomBGColorToggle,
						passValue = group,
					},
					setColor = {
						type = "color",
						name = "Color",
						desc = "Set the custom background color for the health bar.",
						get = GetCustomBGColor,
						set = SetCustomBGColor,
						disabled = function() return not GetCustomBGColorToggle(group) end,
						passValue = group,
					},
				},
			},
			happinessColor = petOptions and {
				name = "Color by happiness",
				desc = "Color health bar by pet happiness status.",
				type = "toggle",
				get = GetPetHappiness,
				set = SetPetHappiness,
			},
			deficit = {
				type = "toggle",
				name = "Show as deficit",
				desc = "Show health deficit, filling the bar when you lose health.",
				get = getDeficit,
				set = setDeficit,
				passValue = group,
			},
			reverse = {
				type = "toggle",
				name = "Reverse",
				desc = "Reverses the direction of the health bar, filling it from right to left.",
				get = getReverse,
				set = setReverse,
				passValue = group,
			}
		}
	}
end)

