local VERSION = tonumber(("$Revision: 33203 $"):match("%d+"))

local PitBull = PitBull
local PitBull_Leader = PitBull:NewModule("Leader", "AceEvent-2.0")
local self = PitBull_Leader
if PitBull.revision < VERSION then
	PitBull.version = "r" .. VERSION
	PitBull.revision = VERSION
	PitBull.date = ("$Date: 2007-04-20 15:43:25 -0400 (Fri, 20 Apr 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local new, del, newHash = PitBull.new, PitBull.del, PitBull.newHash
local newFrame, delFrame = PitBull.newFrame, PitBull.delFrame

PitBull_Leader:RegisterPitBullChildFrames('leaderIcon')

function PitBull_Leader:OnInitialize()
	self.db = PitBull:AcquireDBNamespace("Leader")
	PitBull:RegisterDefaults("Leader", "profile", {
		['*'] = {
			disable = false,
		},
	})
end

function PitBull_Leader:OnEnable()
	self:RegisterEvent("PARTY_LEADER_CHANGED")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED", "PARTY_LEADER_CHANGED")
	self:RegisterEvent("RAID_ROSTER_UPDATE", "PARTY_LEADER_CHANGED")
end

function PitBull_Leader:Update(unit, frame)
	if frame.group ~= "player" and frame.group ~= "party" and frame.group ~= "raid" then
		return
	end
	local has = true
	if unit == "player" then
		if not IsPartyLeader() then
			has = false
		end
	else
		local raid = unit:match("^raid(%d%d?)$")
		if raid then
			local _, rank = GetRaidRosterInfo(tonumber(raid))
			if rank ~= 2 then
				has = false
			end
		else
			local party = unit:match("^party(%d)$")
			if party then
				if GetPartyLeaderIndex() ~= tonumber(party) then
					has = false
				end
			else
				has = false
			end
		end
	end
	if PitBull.configMode and not UnitExists(unit) then
		has = true
	end
	if self.db.profile[frame.group].disable then
		has = false
	end
	
	if has then
		if not frame.leaderIcon then
			local leaderIcon = newFrame("Texture", frame.overlay, "ARTWORK")
			frame.leaderIcon = leaderIcon
			leaderIcon:SetTexture("Interface\\GroupFrame\\UI-Group-LeaderIcon")
			
			PitBull:UpdateLayout(frame)
		end
	else
		if frame.leaderIcon then
			frame.leaderIcon = delFrame(frame.leaderIcon)
			
			PitBull:UpdateLayout(frame)
		end
	end
end

function PitBull_Leader:PARTY_LEADER_CHANGED()
	for unit, frame in PitBull:IterateNonWackyUnitFrames() do
		self:Update(unit, frame)
	end
end

function PitBull_Leader:OnUpdateFrame(unit, frame)
	self:Update(unit, frame)
end

function PitBull_Leader:OnClearUnitFrame(unit, frame)
	if frame.leaderIcon then
		frame.leaderIcon = delFrame(frame.leaderIcon)
	end
end

function PitBull_Leader:OnUnknownLayout(unit, frame)
	frame.leaderIcon:SetPoint("CENTER", frame, "TOPLEFT", 0, 0)
	frame.leaderIcon:SetWidth(12)
	frame.leaderIcon:SetHeight(12)
end

local function getDisable(group)
	return PitBull_Leader.db.profile[group].disable
end
local function setDisable(group, value)
	PitBull_Leader.db.profile[group].disable = value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_Leader:Update(unit, frame)
	end
end
PitBull_Leader:RegisterPitBullOptionsMethod(function(group)
	if group == "player" or group == "party" or group == "raid" then
		return {
			type = "toggle",
			name = "Hide Leader Icon",
			desc = "Hides the leader icon, indicating who is the current group or raid leader.",
			get = getDisable,
			set = setDisable,
			passValue = group,
		}
	end
end)
